/*
 * KFailure.cpp
 *
 *  Created on: Mar 30, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "KFailure.h"
#include <sstream>

KFailure::KFailure(unsigned int k = 1, double effectiveness=0.5) :
	RolloutMonteCarlo(effectiveness, 1.0),
	mK(k)
{ }

KFailure::~KFailure()
{ }

action_t KFailure::select(GameState& state) {
	return state.searchData.customAction;
}

void KFailure::update(GameState& state, double resultValue) {
	RolloutMonteCarlo::update(state, resultValue);
	if(state.isTerminal()) return; //doesn't matter

	bool isMin = state.getPlayer() == A;
	if((isMin && resultValue > .5) || (!isMin && resultValue < .5)) {
		unsigned int& failures = state.searchData.customUInt;
		if(failures + 1 >= mK) {
			action_t& nextAction = state.searchData.customAction;
			nextAction = (1 + nextAction) % state.countChildren();
			failures = 0;
		} else {
			failures++;
		}
	}
}

action_t KFailure::currentBestMove() {
	//return mTree->getRoot().searchData.customAction;
	return RolloutMonteCarlo::currentBestMove();
}

std::string KFailure::toString() const {
	std::stringstream out;
	out << mK << "-failure";
	return out.str();
}
