/*
 * UCTTuned.cpp
 *
 *  Created on: 9 apr 2010
 *      Author: giraffe
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "UCTTuned.h"

UCTTuned::UCTTuned(double effectiveness, double propogation, double fpu):
	UCTV(effectiveness, propogation, fpu)
{ }

double UCTTuned::explorationBias(GameState& state) {
	GameState& parent = *state.getParent();
	SearchData& data = state.searchData;

	// Bias term which will increase as this node goes unexplored.
	double biasTerm = log(parent.searchData.numVisits+1)/(data.numVisits+1);

	// Get the upper bound of the variance...
	double upperVarianceBound = data.estimatedValue +
			sqrt(2*biasTerm);

	// Set it to 1/4 if the upper bound is too low, so as to ensure more exploration.
	upperVarianceBound = upperVarianceBound < .25 ? .25 : upperVarianceBound;

	// Return the bias!
	return sqrt(biasTerm*upperVarianceBound);

}

std::string UCTTuned::toString() const {
	return "uct-tuned";
}

UCTTuned::~UCTTuned() {
	// TODO Auto-generated destructor stub
}
