/*
 * DataFormatter.h
 *
 *  Created on: 2010.03.22.
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef DATAFORMATTER_H_
#define DATAFORMATTER_H_

#include "../search/Searcher.h"
#include <ostream>

/**
 * A DataFormatter takes in Searchers and formats search statistics such that they can
 * be used in higher level processing with greater ease.
 */
class DataFormatter {
public:

	/**
	 * Constructs a DataFormatter that will produce output on the given stream.
	 *
	 * @param out The output stream in which to place Searcher data.
	 */
	DataFormatter(std::ostream& out);

	/**
	 * Parses a given Searcher and outputs statistics to the DataFormatter's stream.
	 *
	 * @param searcher The searcher from which to gather data.
	 */
	void parse(Searcher& searcher);

	virtual ~DataFormatter();

private:

	// The output stream to use when parsing Searchers.
	std::ostream* output;
};

#endif /* DATAFORMATTER_H_ */
