/*
 * GameTree.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "GameTree.h"
#include "GameTreeGenerator.h"
#include <exception>

GameTree::GameTree(GameTreeGenerator& generator, GameState* root) :
	mGenerator(generator),
	mRoot(root),
	mCurrent(root)
{ }

GameTree::~GameTree()
{
	delete mRoot;
}

void GameTree::reset() {
	mCurrent = mRoot;
	mGenerator.reset();
	mRoot->resetChildArray();
}
bool GameTree::isFinished() const {
	return mCurrent->isTerminal();
}

GameState & GameTree::getRoot() const {
	return *mRoot;
} // GameTree::getRoot()

GameState & GameTree::getCurrent() const {
	return *mCurrent;
} // GameTree::getCurrent()

void GameTree::play(action_t index) {
	mCurrent = &(mCurrent->getChild(index));
} // GameTree::play(size_t)

unsigned long GameTree::cacheSize() const {
	return mGenerator.cacheSize();
}
