/*
 * GameTreeGenerator.h
 *
 *  Created on: Mar 22, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef GAMETREEGENERATOR_H_
#define GAMETREEGENERATOR_H_

#include "MinimaxGameState.h"
#include "GameTree.h"
#include "MemoryPool.h"

typedef MinimaxGameState GameStateType;

/**
 * Creates new gametrees from scratch
 */
class GameTreeGenerator {
private:
	action_t mBranchingFactor;
	unsigned int mMaxDepth;
	double mDifficultyBias;
	RandomGenerator mGenerator;

	MemoryPool mStatePool;
	MemoryPool mChildArrayPool;
public:
	GameTreeGenerator(action_t branchingFactor, unsigned int maxDepth, double difficultyBias, uint32_t seed, unsigned int blockSize);
	~GameTreeGenerator();

	RandomGenerator & getRNG();
	unsigned int getMaxDepth() const;
	GameTree* generate();
	/**
	 * Returns a pointer to a block of memory which can be used to hold the next
	 * game state
	 */
	void* nextState();

	/**
	 * Returns a pointer to a block of memory big enough for a GameState*[branchingFactor]
	 */
	void* nextChildArray();

	/**
	 * Resets the memory pool to allow reuse
	 */
	void reset();


	/**
	 * @return The number of nodes in the generated cache so far
	 */
	unsigned long cacheSize() const;
};

#endif /* GAMETREEGENERATOR_H_ */
