/*
 * MemoryPool.h
 *
 *  Created on: Mar 26, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef MEMORYPOOL_H_
#define MEMORYPOOL_H_

#include <stdlib.h>
#include <list>

/*
 *
 */
class MemoryPool {
private:
	/** Bytes per object */
	size_t mObjectSize;
	/** Objects per block */
	unsigned int mBlockSize;
	/** Do objects need to be zero'd out before they can be reused? */
	bool mZeroOnFree;

	std::list<void*> mBlocks;
	unsigned int mNumBlocks;
	unsigned int mNextStateIndex;
	unsigned int mNextBlockIndex;
	void* mCurrentBlock;
	std::list<void*>::iterator mBlocksIterator;

public:
	MemoryPool(size_t objectSize, unsigned int blockSize, bool zeroOnFree);
	virtual ~MemoryPool();

	void* next();
	void freeAll();

	/**
	 * @return Number of objects that have been allocated (might be taking up
	 * more space than that * objectSize)
	 */
	unsigned long size() const;
};

#endif /* MEMORYPOOL_H_ */
