#!/bin/bash

source $(dirname $0)/gombaheader.sh

echo "----- Generating condor submission file -----"
REMOTE_EXECUTABLE=$CLUSTER_PATH/$GOMBA_DIR/$BUILD_CONFIG/$GOMBA_EXECUTABLE
LOCAL_SUBM_FILE=$LOCAL_PATH/$GOMBA_DIR/$BUILD_CONFIG/$SUBM_FILE
python generate_subm.py $CLUSTER_PATH/$RESULTS_DIR $REMOTE_EXECUTABLE $* > $LOCAL_SUBM_FILE
error_exit "Failed to generate subm file!" 
echo "----- Submission file generated. First $SUBM_HEADER_LINES are: -----"
SUBM_HEADER_LINES=25
head -n$SUBM_HEADER_LINES $LOCAL_SUBM_FILE

echo ""

echo "----- Beginning rsync of generated results folders -----"
cluster_run "rm -rf $CLUSTER_PATH/$RESULTS_DIR"
error_exit "Failed to remove old results dir!" 
scp -r ./$RESULTS_DIR $CLUSTER_FULLHOST:$CLUSTER_PATH
#rsync -avz -e ssh ./$RESULTS_DIR $CLUSTER_FULLHOST:$CLUSTER_PATH
error_exit "Failed to scp results directory tree!" 
echo "----- Result folder rsync complete -----"

echo ""

echo "----- Beginning rsync of local build to cluster -----"
rsync -avz -e ssh $LOCAL_PATH/$GOMBA_DIR $CLUSTER_FULLHOST:$CLUSTER_PATH
error_exit "Failed to rsync build directory!"
echo "----- rsync complete -----"

echo ""

FULL_BUILDPATH="$CLUSTER_PATH/$GOMBA_DIR/$BUILD_CONFIG"
SSHINIT="cd $CLUSTER_PATH/$GOMBA_DIR/$BUILD_CONFIG"
echo "----- Building project (config: $BUILD_CONFIG) on server -----"
cluster_run "$SSHINIT && make clean && make"
error_exit "Failed to build gomba ($BUILD_CONFIG)!"
echo "----- Completed build -----"

echo ""

echo "----- Submitting condor job -----"
cluster_submit "$FULL_BUILDPATH/$SUBM_FILE" 30
error_exit "Failed to submit jobs to condor!"
echo "----- Condor submission complete -----"

echo "----- Starting result parser -----"
exec $(dirname $0)/parsegomba.sh