/*
 * Searcher.cpp
 *
 *  Created on: 2010.03.19.
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Searcher.h"
#include "SearchAlgorithm.h"
#include <iostream>

Searcher::Searcher(std::string name, SearchAlgorithm& alg, unsigned int maxIterations):
	mName(name),
	mAlgorithm(&alg),
	mMaxIterations(maxIterations),
	mResults(maxIterations){
}

void Searcher::play(GameTree& tree) {

	mAlgorithm->reset(tree);
	action_t lastAction = 0;
	player_t lastWinner = B;
	int numConsecutive = 0;
	clock_t startTime = clock();

	for(int i = 0; i < mMaxIterations ; i++) {
		mAlgorithm->iterate();

		clock_t endTime = clock();

		action_t chosenAction = mAlgorithm->currentBestMove();
		GameState& chosenState = tree.getCurrent().getChild(chosenAction);

		if(chosenState.getWinner() == A) {
			mResults[i].wins += 1;
		}

		clock_t duration = endTime - startTime;

		mResults[i].totalDifficulty += chosenState.getDifficulty();
		mResults[i].totalDuration += duration;
		mResults[i].nodesExpanded += tree.cacheSize();

		if(lastAction == chosenAction) {
			numConsecutive++;
			//std::cout << numConsecutive << " consecutive plays.\n";
			//if(lastWinner != A) std::cout << "PROBLEM\n";
		} else {
			numConsecutive = 0;
		}

		lastAction = chosenAction;
		lastWinner = chosenState.getWinner();
	}

} // Searcher::play(GameTree&)


std::vector<ResultData>& Searcher::getResults() {
	return mResults;
}
Searcher::~Searcher() {

} // Searcher::~Searcher()

std::string Searcher::getIdentifier() {
	return mName;
}
