/*
 * RandomGeneratorTest.cpp
 *
 *  Created on: Mar 29, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "../gametree/MemoryPool.h"
#include <gtest/gtest.h>

namespace {

class MemoryPoolTest : public ::testing::Test {
};

TEST_F(MemoryPoolTest, CanAccessValues) {
	MemoryPool mp(sizeof(double), 100, false);
	double* d;
	double* prev = NULL;
	for(int i=0;i<272727;i++) {
		d = (double*) mp.next();
		ASSERT_NE(d, prev);
		prev = d;
		*d = i+3;
	}
}

TEST_F(MemoryPoolTest, StartsZeroed) {
	const unsigned int arraySize = 10;
	MemoryPool mp(sizeof(char) * arraySize, 100, true);
	char* currentArray;

	for(unsigned int i=0;i<272727;i++) {
		currentArray = (char*) mp.next();
		for(unsigned int j=0;j<arraySize;j++)
			ASSERT_EQ(currentArray[j], 0);
	}
}

TEST_F(MemoryPoolTest, Resets) {
	MemoryPool mp(sizeof(long), 100, true);
	long* l = (long*) mp.next();
	*l = 27;
	ASSERT_EQ(1, mp.size());

	mp.freeAll();
	ASSERT_EQ(0, mp.size());

	l = (long*) mp.next();
	*l = 42;
	ASSERT_EQ(1, mp.size());
}

TEST_F(MemoryPoolTest, ResetZeroes) {
	MemoryPool mp(sizeof(long), 100, true);
	long* l = (long*) mp.next();
	*l = 27;
	ASSERT_EQ(1, mp.size());

	mp.freeAll();
	ASSERT_EQ(0, mp.size());

	l = (long*) mp.next();
	ASSERT_EQ(1, mp.size());
	ASSERT_EQ(*l, 0);
}

TEST_F(MemoryPoolTest, RepeatedAccess) {
	MemoryPool mp(sizeof(long), 128, false);
	long* l;

	for(int i=0;i<100;i++) {
		for(int j=0;j<500+4*i;j++) {
			ASSERT_EQ(j, mp.size());
			l = (long*) mp.next();
		}
		mp.freeAll();
	}
}

TEST_F(MemoryPoolTest, RepeatedAccessZeroes) {
	MemoryPool mp(sizeof(long), 128, true);
	long* l;

	for(int i=0;i<100;i++) {
		for(int j=0;j<500+4*i;j++) {
			ASSERT_EQ(j, mp.size());
			l = (long*) mp.next();
			ASSERT_EQ(0, *l);
		}
		mp.freeAll();
	}
}

}
