/*
 * TreeTest.cpp
 *
 *  Created on: Mar 29, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "../gametree/GameTreeGenerator.h"
#include <time.h>
#include <gtest/gtest.h>

namespace {

class TreeGenerationSpeedTest : public ::testing::Test {
};

TEST_F(TreeGenerationSpeedTest, BreadthFirstDepth3) {
	unsigned short branching = 126; // cube root of 2mil
	unsigned int nodesToGenerate = branching * branching * branching;

	unsigned int searchDepth = 3;
	unsigned int maxDepth = searchDepth + 10;

	GameState *s1, *s2;

	GameTreeGenerator treeGenerator(branching, maxDepth, .5, time(NULL), 1000);
	GameTree* tree = treeGenerator.generate();
	GameState& root = tree->getRoot();

	clock_t startTime = clock();

	unsigned short level1, level2, level3;
	for(level1 = 0; level1 < branching ; level1++) {
		s1 = &(root.getChild(level1));
		for(level2 = 0; level2 < branching ; level2++) {
			s2 = &(s1->getChild(level2));
			for(level3 = 0; level3 < branching ; level3++) {
				s2->getChild(level3);
			}
		}
		if(clock() - startTime >= CLOCKS_PER_SEC) break;
	}

	clock_t endTime = clock();
	double duration = (endTime - startTime) / ((double) CLOCKS_PER_SEC);

	std::cout << "Generated " << (level1 * branching * branching) << " nodes in " << duration << "seconds."  << std::endl;
}

TEST_F(TreeGenerationSpeedTest, MonteCarloSpeed) {
	unsigned int nodesToGenerate = 2000000;
	unsigned short branching = 200;
	unsigned int depth       = 100;

	unsigned int paths = nodesToGenerate/depth;

	RandomGenerator rng;

	GameTreeGenerator treeGenerator(branching, depth, .5, time(NULL), 1000);
	GameTree* tree = treeGenerator.generate();
	GameState* state;

	clock_t startTime = clock();

	unsigned int p;
	for(p=0;p<paths;p++) {
		state = &(tree->getRoot());
		for(unsigned int d=0;d<depth;d++) {
			state = &(state->getChild(rng.nextAction(branching)));
		}

		if(p % 256 == 0 && clock() - startTime >= CLOCKS_PER_SEC) break;
	}

	clock_t endTime = clock();
	double duration = (endTime - startTime) / ((double) CLOCKS_PER_SEC);

	std::cout << "Generated " << (p*depth) << " nodes in " << duration << "seconds." << std::endl;
}

} // namespace
